/// <reference types="node" />
/// <reference types="ws" />
import { EventEmitter } from 'events';
import { Socket } from 'net';
import IsomorphicWebSocket from 'isomorphic-ws';
import { Agent } from 'http';
export declare const relayEvent: unique symbol;
export declare const isNode = true;
export declare type LiveOptions = {
    protover?: 1 | 2 | 3;
    key?: string;
};
export declare type WSOptions = LiveOptions & {
    address?: string;
    agent?: Agent;
};
export declare type TCPOptions = LiveOptions & {
    host?: string;
    port?: number;
};
declare class WebSocket extends EventEmitter {
    ws: IsomorphicWebSocket;
    constructor(address: string, ...args: any[]);
    get readyState(): 0 | 2 | 3 | 1;
    send(data: Buffer): void;
    close(code?: number, data?: string): void;
}
declare class NiceEventEmitter extends EventEmitter {
    emit(eventName: string | symbol, ...params: any[]): boolean;
}
declare class Live extends NiceEventEmitter {
    roomid: number;
    online: number;
    live: boolean;
    closed: boolean;
    timeout: ReturnType<typeof setTimeout>;
    send: (data: Buffer) => void;
    close: () => void;
    constructor(roomid: number, { send, close, protover, key }: {
        send: (data: Buffer) => void;
        close: () => void;
    } & LiveOptions);
    heartbeat(): void;
    getOnline(): Promise<number>;
}
export declare class LiveWS extends Live {
    ws: InstanceType<typeof WebSocket>;
    constructor(roomid: number, { address, protover, key, agent }?: WSOptions);
}
export declare class LiveTCP extends Live {
    socket: Socket;
    buffer: Buffer;
    i: number;
    constructor(roomid: number, { host, port, protover, key }?: TCPOptions);
    splitBuffer(): void;
}
declare class KeepLive<Base extends typeof LiveWS | typeof LiveTCP> extends EventEmitter {
    params: [number, any?];
    closed: boolean;
    interval: number;
    timeout: number;
    connection: InstanceType<typeof LiveWS | typeof LiveTCP>;
    Base: Base;
    constructor(Base: Base, ...params: ConstructorParameters<Base>);
    connect(reconnect?: boolean): void;
    get online(): number;
    get roomid(): number;
    close(): void;
    heartbeat(): void;
    getOnline(): Promise<number>;
    send(data: Buffer): void;
}
export declare class KeepLiveWS extends KeepLive<typeof LiveWS> {
    constructor(...params: ConstructorParameters<typeof LiveWS>);
}
export declare class KeepLiveTCP extends KeepLive<typeof LiveTCP> {
    constructor(...params: ConstructorParameters<typeof LiveTCP>);
}
export {};
